@@ -37,8 +37,10 @@ info = "'rethinkdb import` loads data into a RethinkDB cluster"
 usage = "\
   rethinkdb import -d DIR [-c HOST:PORT] [-a AUTH_KEY] [--force]\n\
       [-i (DB | DB.TABLE)] [--clients NUM]\n\
+      [--shards NUM_SHARDS] [--replicas NUM_REPLICAS]\n\
   rethinkdb import -f FILE --table DB.TABLE [-c HOST:PORT] [-a AUTH_KEY]\n\
       [--force] [--clients NUM] [--format (csv | json)] [--pkey PRIMARY_KEY]\n\
+      [--shards NUM_SHARDS] [--replicas NUM_REPLICAS]\n\
       [--delimiter CHARACTER] [--custom-header FIELD,FIELD... [--no-header]]"
 
 def print_import_help():
@@ -113,6 +115,10 @@ def parse_options():
     parser.add_option("--debug", dest="debug", action="store_true", default=False)
     parser.add_option("--max-document-size", dest="max_document_size",  default=0,type="int")
 
+    # Replication settings
+    parser.add_option("--shards", dest="shards", metavar="NUM_SHARDS", default=0, type="int")
+    parser.add_option("--replicas", dest="replicas", metavar="NUM_REPLICAS", default=0, type="int")
+
     # Directory import options
     parser.add_option("-d", "--directory", dest="directory", metavar="DIRECTORY", default=None, type="string")
     parser.add_option("-i", "--import", dest="tables", metavar="DB | DB.TABLE", default=[], action="append", type="string")
@@ -152,6 +158,11 @@ def parse_options():
     res["debug"] = options.debug
     res["create_sindexes"] = options.create_sindexes
 
+    res["create_args"] = { }
+    for k in ['shards', 'replicas']:
+        if getattr(options, k) != 0:
+            res["create_args"][k] = getattr(options, k)
+
     # Default behavior for csv files - may be changed by options
     res["delimiter"] = ","
     res["no_header"] = False
@@ -261,7 +272,8 @@ def parse_options():
             if options.custom_header is not None:
                 raise RuntimeError("Error: --custom-header option is only valid for CSV file formats")
 
-        res["primary_key"] = options.primary_key
+        if options.primary_key is not None:
+          res["create_args"]["primary_key"] = options.primary_key
     else:
         raise RuntimeError("Error: Must specify one of --directory or --file to import")
 
@@ -514,9 +526,11 @@ def csv_reader(task_queue, filename, db, table, options, progress_info, exit_eve
 
 # This function is called through rdb_call_wrapper, which will reattempt if a connection
 # error occurs.  Progress will resume where it left off.
-def create_table(progress, conn, db, table, pkey, sindexes):
-    if table not in r.db(db).table_list().run(conn):
-        r.db(db).table_create(table, primary_key=pkey).run(conn)
+def create_table(progress, conn, db, table, create_args, sindexes):
+    # Make sure that the table is ready if it exists, or create it
+    r.branch(r.db(db).table_list().contains(table),
+        r.db(db).table(table).wait(timeout=30),
+        r.db(db).table_create(table, **create_args)).run(conn)
 
     if progress[0] is None:
         progress[0] = 0
@@ -524,21 +538,25 @@ def create_table(progress, conn, db, table, pkey, sindexes):
     # Recreate secondary indexes - assume that any indexes that already exist are wrong
     # and create them from scratch
     indexes = r.db(db).table(table).index_list().run(conn)
+    created_indexes = list()
     for sindex in sindexes[progress[0]:]:
         if isinstance(sindex, dict) and all(k in sindex for k in ('index', 'function')):
             if sindex['index'] in indexes:
                 r.db(db).table(table).index_drop(sindex['index']).run(conn)
             r.db(db).table(table).index_create(sindex['index'], sindex['function']).run(conn)
+            created_indexes.append(sindex['index'])
         progress[0] += 1
+    r.db(db).table(table).index_wait(r.args(created_indexes)).run(conn)
 
 def table_reader(options, file_info, task_queue, error_queue, progress_info, exit_event):
     try:
         db = file_info["db"]
         table = file_info["table"]
-        primary_key = file_info["info"]["primary_key"]
+        create_args = dict(options["create_args"])
+        create_args["primary_key"] = file_info["info"]["primary_key"]
 
         conn_fn = lambda: r.connect(options["host"], options["port"], auth_key=options["auth_key"])
-        rdb_call_wrapper(conn_fn, "create table", create_table, db, table, primary_key,
+        rdb_call_wrapper(conn_fn, "create table", create_table, db, table, create_args,
                          file_info["info"]["indexes"] if options["create_sindexes"] else [])
 
         if file_info["format"] == "json":
@@ -789,7 +807,9 @@ def import_directory(options):
 
     spawn_import_clients(options, files_info)
 
-def table_check(progress, conn, db, table, pkey, force):
+def table_check(progress, conn, db, table, create_args, force):
+    pkey = None
+
     if db == "rethinkdb":
         raise RuntimeError("Error: Cannot import a table into the system database: 'rethinkdb'")
 
@@ -800,29 +820,29 @@ def table_check(progress, conn, db, table, pkey, force):
         if not force:
             raise RuntimeError("Error: Table already exists, run with --force if you want to import into the existing table")
 
-        extant_pkey = r.db(db).table(table).info().run(conn)["primary_key"]
-        if pkey is not None and pkey != extant_pkey:
-            raise RuntimeError("Error: Table already exists with a different primary key")
-        pkey = extant_pkey
+        if 'primary_key' in create_args:
+            pkey = r.db(db).table(table).info()["primary_key"].run(conn)
+            if create_args["primary_key"] != pkey:
+                raise RuntimeError("Error: Table already exists with a different primary key")
     else:
-        if pkey is None:
-            print("no primary key specified, using default primary key when creating table")
-            r.db(db).table_create(table).run(conn)
+        if 'primary_key' in create_args:
+            pkey = create_args["primary_key"]
         else:
-            r.db(db).table_create(table, primary_key=pkey).run(conn)
+            print("no primary key specified, using default primary key when creating table")
+        r.db(db).table_create(table, **create_args).run(conn)
+
     return pkey
 
 def import_file(options):
     db = options["import_db_table"][0]
     table = options["import_db_table"][1]
-    pkey = options["primary_key"]
 
     # Ensure that the database and table exist with the right primary key
     conn_fn = lambda: r.connect(options["host"], options["port"], auth_key=options["auth_key"])
     # Make sure this isn't a pre-`reql_admin` cluster - which could result in data loss
     # if the user has a database named 'rethinkdb'
     rdb_call_wrapper(conn_fn, "version check", check_minimum_version, (1, 16, 0))
-    pkey = rdb_call_wrapper(conn_fn, "table check", table_check, db, table, pkey, options["force"])
+    pkey = rdb_call_wrapper(conn_fn, "table check", table_check, db, table, options["create_args"], options["force"])
 
     # Make this up so we can use the same interface as with an import directory
     file_info = {}
