@@ -1,28 +1,24 @@
 # Copyright 2010-2015 RethinkDB, all rights reserved.
 
-__all__ = ["ReqlCursorEmpty",
-           "ReqlError",
-           "ReqlCompileError",
-           "ReqlRuntimeError",
-           "ReqlQueryLogicError",
-           "ReqlNonExistenceError",
-           "ReqlResourceLimitError",
-           "ReqlUserError",
-           "ReqlInternalError",
-           "ReqlTimeoutError",
-           "ReqlAvailabilityError",
-           "ReqlOpFailedError",
-           "ReqlOpIndeterminateError",
-           "ReqlDriverError",
-           "ReqlAuthError",
-
-           "RqlCursorEmpty",
-           "RqlError", 
-           "RqlClientError",
-           "RqlCompileError",
-           "RqlRuntimeError",
-           "RqlDriverError",
-           "RqlTimeoutError"]
+__all__ = [
+    "ReqlCursorEmpty", "RqlCursorEmpty",
+    "ReqlError", "RqlError",
+        "ReqlCompileError", "RqlCompileError",
+            "ReqlDriverCompileError",
+            "ReqlServerCompileError",
+        "ReqlRuntimeError", "RqlRuntimeError",
+            "ReqlQueryLogicError",
+                "ReqlNonExistenceError",
+            "ReqlResourceLimitError",
+            "ReqlUserError",
+            "ReqlInternalError",
+            "ReqlTimeoutError", "RqlTimeoutError",
+            "ReqlAvailabilityError",
+                "ReqlOpFailedError",
+                "ReqlOpIndeterminateError",
+        "ReqlDriverError", "RqlDriverError", "RqlClientError",
+            "ReqlAuthError"
+]
 
 import sys
 
@@ -81,6 +77,12 @@ class ReqlCompileError(ReqlError):
 
 RqlCompileError = ReqlCompileError
 
+class ReqlDriverCompileError(ReqlCompileError):
+    pass
+
+class ReqlServerCompileError(ReqlCompileError):
+    pass
+
 
 class ReqlRuntimeError(ReqlError):
     pass
